﻿using IRaCIS.Core.Application.Contracts;
using IRaCIS.Core.Domain.Share;
using Microsoft.AspNetCore.Mvc;

namespace IRaCIS.Core.Application
{
    [ApiExplorerSettings(GroupName = "Trial")]
    public class PersonalWorkstation : BaseService
    {
        private readonly IRepository<Trial> _trialRepository;
        private readonly IRepository<TrialUser> _trialUserRepository;
        private readonly IRepository<TrialDocument> _trialDocumentRepository;
        private readonly IRepository<SystemDocument> _systemDocumentRepository;
        private readonly IRepository<SystemNotice> _systemNoticeRepository;

        public PersonalWorkstation(IRepository<Trial> trialRepository, IRepository<TrialUser> trialUserRepository, IRepository<TrialDocument> trialDocumentRepository,
            IRepository<SystemDocument> systemDocumentRepository, IRepository<SystemNotice> systemNoticeRepository)
        {
            _trialRepository = trialRepository;
            _trialUserRepository = trialUserRepository;
            _trialDocumentRepository = trialDocumentRepository;
            _systemDocumentRepository = systemDocumentRepository;
            _systemNoticeRepository = systemNoticeRepository;
        }

        /// <summary>
        /// 个人面板 统计值
        /// </summary>
        /// <returns></returns>
        public async Task<PersonalStataDTO> GetBasicStat()
        {

            return new PersonalStataDTO()
            {
                //正参与的数量
                TrialCount = _userInfo.UserTypeEnumInt == (int)UserTypeEnum.SuperAdmin
                    ? await _trialRepository.CountAsync()
                    : await _trialUserRepository.Where(t => t.UserId == _userInfo.Id).CountAsync(),

                DeletedCount = _userInfo.UserTypeEnumInt == (int)UserTypeEnum.SuperAdmin
                    ? await _trialRepository.AsQueryable(true).CountAsync(t => t.IsDeleted)
                    : await _trialUserRepository.AsQueryable(true).Where(t => t.UserId == _userInfo.Id && t.IsDeleted)
                        .CountAsync(),


                TotalNeedSignTrialDocCount = _userInfo.UserTypeEnumInt == (int)UserTypeEnum.SuperAdmin
                    ? 0
                    : await _trialDocumentRepository.AsQueryable(true).Where(t => t.Trial.TrialStatusStr != StaticData.TrialState.TrialStopped)
                        .Where(t => t.Trial.TrialUserList.Any(t => t.UserId == _userInfo.Id))
                        .Where(t => t.IsDeleted == false || (t.IsDeleted == true && t.TrialDocConfirmedUserList.Any(t => t.ConfirmUserId == _userInfo.Id && t.ConfirmTime != null)))

                        .SelectMany(t => t.NeedConfirmedUserTypeList)
                        .CountAsync(t => t.NeedConfirmUserTypeId == _userInfo.UserTypeId),

                HaveSignedTrialDocCount = _userInfo.UserTypeEnumInt == (int)UserTypeEnum.SuperAdmin
                    ? 0
                    //废除了 已经签署了也要算进去
                    : await _trialDocumentRepository.AsQueryable(true).Where(t => t.Trial.TrialStatusStr != StaticData.TrialState.TrialStopped)
                        .Where(t => t.Trial.TrialUserList.Any(t => t.UserId == _userInfo.Id))
                        .Where(t => t.NeedConfirmedUserTypeList.Any(t => t.NeedConfirmUserTypeId == _userInfo.UserTypeId))
                        .SelectMany(t => t.TrialDocConfirmedUserList)
                        .CountAsync(t => t.ConfirmUserId == _userInfo.Id && t.ConfirmTime != null),

                TotalNeedSignSystemDocCount = _userInfo.UserTypeEnumInt == (int)UserTypeEnum.SuperAdmin
                    ? 0
                    : await _systemDocumentRepository
                        .Where(t => t.IsDeleted == false || (t.IsDeleted == true && t.SystemDocConfirmedUserList.Any(t => t.ConfirmUserId == _userInfo.Id && t.ConfirmTime != null)))
                        .SelectMany(t => t.NeedConfirmedUserTypeList)
                        .CountAsync(t => t.NeedConfirmUserTypeId == _userInfo.UserTypeId),


                HaveSignedSystemDocCount = _userInfo.UserTypeEnumInt == (int)UserTypeEnum.SuperAdmin
                    ? 0
                    : await _systemDocumentRepository
                        .Where(t => t.NeedConfirmedUserTypeList.Any(t => t.NeedConfirmUserTypeId == _userInfo.UserTypeId))
                        .SelectMany(t => t.SystemDocConfirmedUserList)
                        .CountAsync(t => t.ConfirmUserId == _userInfo.Id && t.ConfirmTime != null),

                TotalApprovalRequiredCount =
                    _userInfo.UserTypeEnumInt == (int)UserTypeEnum.ProjectManager || _userInfo.UserTypeEnumInt == (int)UserTypeEnum.APM ?
                    _trialRepository.Where(t => t.TrialUserList.Any(t => t.UserId == _userInfo.Id)).SelectMany(t => t.TrialSiteSurveyList).Where(t => t.State == TrialSiteSurveyEnum.SPMApproved).Count()
                    : _userInfo.UserTypeEnumInt == (int)UserTypeEnum.CPM || _userInfo.UserTypeEnumInt == (int)UserTypeEnum.SPM
                    ? _trialRepository.Where(t => t.TrialUserList.Any(t => t.UserId == _userInfo.Id)).SelectMany(t => t.TrialSiteSurveyList).Where(t => t.State == TrialSiteSurveyEnum.CRCSubmitted).Count()
                    : 0,


                TotalSystemNoticeCount = _userInfo.UserTypeEnumInt == (int)UserTypeEnum.SuperAdmin
                    ? 0 : await _systemNoticeRepository.Where(t => t.NoticeUserTypeList.Any(t => t.UserTypeId == _userInfo.UserTypeId) && t.NoticeStateEnum == Domain.Share.Management.SystemNotice_NoticeStateEnum.HavePublished)

                    .CountAsync(),

                NeedReadSystemNoticeCount = _userInfo.UserTypeEnumInt == (int)UserTypeEnum.SuperAdmin
                    ? 0 : await _systemNoticeRepository.Where(t => t.NoticeUserTypeList.Any(t => t.UserTypeId == _userInfo.UserTypeId) && t.NoticeStateEnum == Domain.Share.Management.SystemNotice_NoticeStateEnum.HavePublished
                     && !t.NoticeUserReadList.Any(t => t.CreateUserId == _userInfo.Id))
                    .Where(t => t.EndDate == null || t.EndDate != null && t.EndDate > DateTime.Now)
                    .CountAsync(),


            };

        }


        /// <summary>
        /// 中心调研    每个项目 需要处理的审批统计
        /// </summary>
        /// <param name="query"></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<PageOutput<TrialSiteSurveyStat>> GetSiteSurveyApprovalList(TrialSiteSurveyStatQuery query)
        {


            if (_userInfo.UserTypeEnumInt == (int)UserTypeEnum.ProjectManager ||
                _userInfo.UserTypeEnumInt == (int)UserTypeEnum.APM ||
                _userInfo.UserTypeEnumInt == (int)UserTypeEnum.CPM ||
                _userInfo.UserTypeEnumInt == (int)UserTypeEnum.SPM)
            {
                return await _trialRepository
                    .Where(t => t.TrialUserList.Any(t => t.UserId == _userInfo.Id))
                    .WhereIf(_userInfo.UserTypeEnumInt == (int)UserTypeEnum.ProjectManager || _userInfo.UserTypeEnumInt == (int)UserTypeEnum.APM, c => c.TrialSiteSurveyList.Where(t => t.State == TrialSiteSurveyEnum.SPMApproved).Count() > 0)
                    .WhereIf(_userInfo.UserTypeEnumInt == (int)UserTypeEnum.CPM || _userInfo.UserTypeEnumInt == (int)UserTypeEnum.SPM, c => c.TrialSiteSurveyList.Where(t => t.State == TrialSiteSurveyEnum.CRCSubmitted).Count() > 0)
                    .ProjectTo<TrialSiteSurveyStat>(_mapper.ConfigurationProvider, new { userTypeEnumInt = _userInfo.UserTypeEnumInt, userId = _userInfo.Id })
                    .OrderByDescending(t => t.ApprovalRequiredCount).ToPagedListAsync(query.PageIndex, query.PageSize, query.SortField, query.Asc);
            }

            else
            {
                return new PageOutput<TrialSiteSurveyStat>(query.PageIndex, query.PageSize, 0, new List<TrialSiteSurveyStat>());

            }

        }


        /// <summary>
        ///   待签署的项目文件      需要签署文件数量  系统级别的在第一行
        /// </summary>
        /// <param name="query"></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<PageOutput<DocSignStat>> GetTrialDocStatList(TrialSiteSurveyStatQuery query)
        {
            if (_userInfo.IsAdmin)
            {
                return new PageOutput<DocSignStat>(query.PageIndex, query.PageSize, 0, new List<DocSignStat>());
            }
            else
            {
                var trialDocStat = await _trialRepository/*.AsQueryable(true)*/.Where(t => t.TrialStatusStr != StaticData.TrialState.TrialStopped)
                    .Where(t => t.TrialUserList.Any(t => t.UserId == _userInfo.Id))
                    .WhereIf(!_userInfo.IsAdmin, c => c.TrialDocumentList.Where(t => t.IsDeleted == false && t.NeedConfirmedUserTypeList.Any(t => t.NeedConfirmUserTypeId == _userInfo.UserTypeId) && !t.TrialDocConfirmedUserList.Any(t => t.ConfirmUserId == _userInfo.Id && t.ConfirmTime != null))
                            .Count() > 0)
                    .ProjectTo<DocSignStat>(_mapper.ConfigurationProvider, new { userTypeEnumInt = _userInfo.UserTypeEnumInt, userId = _userInfo.Id, userTypeId = _userInfo.UserTypeId })
                    .OrderByDescending(t => t.WaitSignCount)
                    .ToPagedListAsync(query.PageIndex, query.PageSize, query.SortField, query.Asc);


                return trialDocStat;
            }

        }


        #region PM  APM 待办
        /// <summary>
        /// 一致性核查待处理
        /// </summary>
        /// <param name="inQuery"></param>
        /// <param name="_subjectVisitRepository"></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<PageOutput<CheckToBeDoneDto>> GetCheckToBeDoneList(CheckToBeDoneQuery inQuery,
            [FromServices] IRepository<SubjectVisit> _subjectVisitRepository,
              [FromServices] IRepository<Trial> _trialRepository)
        {

            var query = _trialRepository
                .Select(t => new CheckToBeDoneDto()
                {
                    TrialId = t.Id,
                    ResearchProgramNo = t.ResearchProgramNo,
                    ExperimentName = t.ExperimentName,
                    TrialCode = t.TrialCode,
                    UrgentCount = t.SubjectVisitList.Where(u => (u.CheckState == CheckStateEnum.ToCheck || u.CheckState == CheckStateEnum.CVIng) && u.IsUrgent).Count(),
                    ToBeCheckedCount = t.SubjectVisitList.Where(u => u.CheckState == CheckStateEnum.ToCheck).Count(),
                    ToBeRepliedCount = t.SubjectVisitList.Where(u => u.CheckState == CheckStateEnum.ToCheck &&
                    u.CheckChallengeDialogList.Any(t => t.UserTypeEnum == UserTypeEnum.ClinicalResearchCoordinator)).Count(),
                });

            var result = await query.ToPagedListAsync(inQuery.PageIndex, inQuery.PageSize, inQuery.SortField, inQuery.Asc);

            return result;

        }

        /// <summary>
        /// 重阅申请待审核
        /// </summary>
        /// <param name="inQuery"></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<PageOutput<ReReadingApplyToBeDoneDto>> GetReReadingApplyToBeDoneList(ReReadingApplyToBeDoneQuery inQuery,
            [FromServices] IRepository<VisitTaskReReading> _visitTaskReReadingRepository,
              [FromServices] IRepository<Trial> _trialRepository)
        {

            var query = _visitTaskReReadingRepository.Where(t => t.OriginalReReadingTask.ReReadingApplyState == ReReadingApplyState.DocotorHaveApplyed)
                .GroupBy(t => new { t.OriginalReReadingTask.Trial.ExperimentName, t.OriginalReReadingTask.Trial.ResearchProgramNo, t.OriginalReReadingTask.Trial.TrialCode, t.OriginalReReadingTask.TrialId })
                 .Select(g => new ReReadingApplyToBeDoneDto()
                 {
                     TrialId = g.Key.TrialId,
                     ResearchProgramNo = g.Key.ResearchProgramNo,
                     ExperimentName = g.Key.ExperimentName,
                     TrialCode = g.Key.TrialCode,
                     UrgentCount = g.Where(u => u.OriginalReReadingTask.IsUrgent == true).Count(),
                     ToBeApprovalCount = g.Count(),

                 });


            var result = await query.ToPagedListAsync(inQuery.PageIndex, inQuery.PageSize, inQuery.SortField, inQuery.Asc);

            return result;

        }


        #endregion


        #region SPM  CPM 待办

        /// <summary>
        /// 阅片人筛选 
        /// </summary>
        /// <param name="inQuery"></param>
        /// <returns></returns>                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              vvv
        [HttpPost]
        public async Task<PageOutput<ReviewerSelectToBeDoneDto>> GetReviewerSelectToBeDoneList(ReviewerSelectToBeDoneQuery inQuery
            , [FromServices] IRepository<Enroll> _enrollRepository,
             [FromServices] IRepository<Trial> _trialRepository)
        {
            var query = _trialRepository
                    .Select(t => new ReviewerSelectToBeDoneDto()
                    {
                        TrialId = t.Id,
                        ResearchProgramNo = t.ResearchProgramNo,
                        ExperimentName = t.ExperimentName,
                        TrialCode = t.TrialCode,

                        IsUrgent = t.IsUrgent || t.IsSubjectExpeditedView,
                        ToBeApprovalCount = t.EnrollList.Where(u => u.EnrollStatus == EnrollStatus.HasCommittedToCRO).Count()
                    });

            var result = await query.ToPagedListAsync(inQuery.PageIndex, inQuery.PageSize, inQuery.SortField, inQuery.Asc);

            return result;

        }

        /// <summary>
        /// 重阅审批
        /// </summary>
        /// <param name="inQuery"></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<PageOutput<ReReadingApprovalToBeDoneDto>> GetReReadingApprovalToBeDoneToBeDoneList(ReReadingApprovalToBeDoneQuery inQuery,
           [FromServices] IRepository<VisitTaskReReading> _visitTaskReReadingRepository,
              [FromServices] IRepository<Trial> _trialRepository)

        {
            var query = _visitTaskReReadingRepository.Where(t => t.OriginalReReadingTask.ReReadingApplyState == ReReadingApplyState.TrialGroupHaveApplyed)
               .GroupBy(t => new { t.OriginalReReadingTask.Trial.ExperimentName, t.OriginalReReadingTask.Trial.ResearchProgramNo, t.OriginalReReadingTask.Trial.TrialCode, t.OriginalReReadingTask.TrialId })
                .Select(g => new ReReadingApprovalToBeDoneDto()
                {
                    TrialId = g.Key.TrialId,
                    ResearchProgramNo = g.Key.ResearchProgramNo,
                    ExperimentName = g.Key.ExperimentName,
                    TrialCode = g.Key.TrialCode,
                    UrgentCount = g.Where(u => u.OriginalReReadingTask.IsUrgent == true).Count(),
                    ToBeApprovalCount = g.Count(),

                });


            var result = await query.ToPagedListAsync(inQuery.PageIndex, inQuery.PageSize, inQuery.SortField, inQuery.Asc);

            return result;

        }

        #endregion


        #region CRC 待办


        /// <summary>
        /// 临床数据
        /// </summary>
        /// <param name="inQuery"></param>
        /// <param name="_subjectVisitRepository"></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<PageOutput<ImageClinicalDataToBeDoneDto>> GetImageClinicalDataToBeDoneList(ImageClinicalDataToBeDoneQuery inQuery,
            [FromServices] IRepository<SubjectVisit> _subjectVisitRepository,
              [FromServices] IRepository<Trial> _trialRepository)
        {

            var query = _trialRepository
                .Where(t=>t.clinicalDataTrialSets.Any(t=>t.ClinicalDataLevel==ClinicalLevel.Subject && t.IsConfirm))
                .Select(t => new ImageClinicalDataToBeDoneDto()
                {
                    TrialId = t.Id,
                    ResearchProgramNo = t.ResearchProgramNo,
                    ExperimentName = t.ExperimentName,
                    TrialCode = t.TrialCode,
                    UrgentCount = t.SubjectVisitList.Where(u =>  u.IsBaseLine&& u.SubmitState != SubmitStateEnum.Submitted && u.IsUrgent ).Count(),
                    ToBeDealedCount = t.SubjectVisitList.Where(u => u.IsBaseLine && u.SubmitState != SubmitStateEnum.Submitted).Count(),

                });

            var result = await query.ToPagedListAsync(inQuery.PageIndex, inQuery.PageSize, inQuery.SortField, inQuery.Asc);

            return result;

        }





        /// <summary>
        /// CRC影像质疑待处理
        /// </summary>
        /// <param name="inQuery"></param>
        /// <param name="_subjectVisitRepository"></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<PageOutput<ImageQuestionToBeDoneDto>> GetCRCImageQuestionToBeDoneList(ImageQuestionToBeDoneQuery inQuery,
            [FromServices] IRepository<SubjectVisit> _subjectVisitRepository,
              [FromServices] IRepository<Trial> _trialRepository)
        {

            var query = _trialRepository
                .Select(t => new ImageQuestionToBeDoneDto()
                {
                    TrialId = t.Id,
                    ResearchProgramNo = t.ResearchProgramNo,
                    ExperimentName = t.ExperimentName,
                    TrialCode = t.TrialCode,
                    UrgentCount = t.SubjectVisitList.SelectMany(c=>c.QCChallengeList)
                    .Where(u => u.IsClosed==false && u.LatestReplyUser.UserTypeEnum==UserTypeEnum.IQC && u.SubjectVisit.IsUrgent).Count(),
                    ToBeDealedCount = t.SubjectVisitList.SelectMany(c => c.QCChallengeList)

                    .Where(u => u.IsClosed == false && u.LatestReplyUser.UserTypeEnum == UserTypeEnum.IQC ).Count(),
                 
                });

            var result = await query.ToPagedListAsync(inQuery.PageIndex, inQuery.PageSize, inQuery.SortField, inQuery.Asc);

            return result;

        }

        /// <summary>
        /// 核查质疑待处理
        /// </summary>
        /// <param name="inQuery"></param>
        /// <param name="_subjectVisitRepository"></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<PageOutput<ImageCheckQuestionToBeDoneDto>> GetImageCheckQuestionToBeDoneList(ImageCheckQuestionToBeDoneQuery inQuery,
            [FromServices] IRepository<SubjectVisit> _subjectVisitRepository,
              [FromServices] IRepository<Trial> _trialRepository)
        {

            var query = _trialRepository
                .Select(t => new ImageCheckQuestionToBeDoneDto()
                {
                    TrialId = t.Id,
                    ResearchProgramNo = t.ResearchProgramNo,
                    ExperimentName = t.ExperimentName,
                    TrialCode = t.TrialCode,
                    UrgentCount = t.SubjectVisitList.Where(u =>  u.CheckState == CheckStateEnum.CVIng && u.CheckChallengeState==CheckChanllengeTypeEnum.PMWaitCRCReply && u.IsUrgent).Count(),
                    ToBeDealedCount = t.SubjectVisitList.Where(u => u.CheckState == CheckStateEnum.CVIng && u.CheckChallengeState == CheckChanllengeTypeEnum.PMWaitCRCReply ).Count()
                    
                });

            var result = await query.ToPagedListAsync(inQuery.PageIndex, inQuery.PageSize, inQuery.SortField, inQuery.Asc);

            return result;

        }


        /// <summary>
        /// 影像重传
        /// </summary>
        /// <param name="inQuery"></param>
        /// <param name="_subjectVisitRepository"></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<PageOutput<ImageReUploadToBeDoneDto>> GetImageReUploadToBeDoneList(ImageReUploadToBeDoneQuery inQuery,
            [FromServices] IRepository<SubjectVisit> _subjectVisitRepository,
              [FromServices] IRepository<Trial> _trialRepository)
        {

            var query = _trialRepository
                .Select(t => new ImageReUploadToBeDoneDto()
                {
                    TrialId = t.Id,
                    ResearchProgramNo = t.ResearchProgramNo,
                    ExperimentName = t.ExperimentName,
                    TrialCode = t.TrialCode,
                    UrgentCount = t.SubjectVisitList.Where(u => u.SubmitState == SubmitStateEnum.ToSubmit && u.IsUrgent && u.IsPMBackOrReReading).Count(),
                    ToBeDealedCount = t.SubjectVisitList.Where(u => u.SubmitState == SubmitStateEnum.ToSubmit && u.IsPMBackOrReReading).Count(),

                });
            
            var result = await query.ToPagedListAsync(inQuery.PageIndex, inQuery.PageSize, inQuery.SortField, inQuery.Asc);

            return result;

        }

        


        /// <summary>
        /// 加急影像提交
        /// </summary>
        /// <param name="inQuery"></param>
        /// <param name="_subjectVisitRepository"></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<PageOutput<ImageSubmittedToBeDoneDto>> GetImageSubmittedToBeDoneList(ImageSubmittedToBeDoneQuery inQuery,
            [FromServices] IRepository<SubjectVisit> _subjectVisitRepository,
              [FromServices] IRepository<Trial> _trialRepository)
        {

            var query = _trialRepository
                .Select(t => new ImageSubmittedToBeDoneDto()
                {
                    TrialId = t.Id,
                    ResearchProgramNo = t.ResearchProgramNo,
                    ExperimentName = t.ExperimentName,
                    TrialCode = t.TrialCode,
                    UrgentCount = t.SubjectVisitList.Where(u => u.SubmitState ==SubmitStateEnum.ToSubmit && u.IsUrgent).Count(),
                    ToBeDealedCount = t.SubjectVisitList.Where(u => u.SubmitState == SubmitStateEnum.ToSubmit ).Count(),

                });

            var result = await query.ToPagedListAsync(inQuery.PageIndex, inQuery.PageSize, inQuery.SortField, inQuery.Asc);

            return result;

        }



        #endregion

        #region IQC待办



        /// <summary>
        /// 影像质控     待领取 跟项目配置有关系 --未完成
        /// </summary>
        /// <param name="inQuery"></param>
        /// <param name="_subjectVisitRepository"></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<PageOutput<ImageQualityToBeDoneDto>> GetImageQualityToBeDoneList(ImageQualityToBeDoneQuery inQuery,
            [FromServices] IRepository<SubjectVisit> _subjectVisitRepository,
              [FromServices] IRepository<Trial> _trialRepository)
        {

            var query = _trialRepository.Where(t=>t.QCProcessEnum != TrialQCProcess.NotAudit)
                .Select(t => new ImageQualityToBeDoneDto()
                {
                    TrialId = t.Id,
                    ResearchProgramNo = t.ResearchProgramNo,
                    ExperimentName = t.ExperimentName,
                    TrialCode = t.TrialCode,

                    // 待审核  加急的（待领取 已领取）--  领取了 还没做 就是待审核
                    UrgentCount = t.SubjectVisitList.Where(u => u.CurrentActionUserId==_userInfo.Id  && u.IsUrgent).Count(),


                    //待领取量
                    ToBeClaimedCount = t.SubjectVisitList.Where(u => u.SubmitState == SubmitStateEnum.ToSubmit).Count(),

                    //已领取 待审核 审核中   (审核完成  领取人就会清理 所以只用查询当前领取人是自己的就好了)
                    ToBeReviewedCount= t.SubjectVisitList.Where(u => u.CurrentActionUserId==_userInfo.Id ).Count()

                });

            var result = await query.ToPagedListAsync(inQuery.PageIndex, inQuery.PageSize, inQuery.SortField, inQuery.Asc);

            return result;

        }


        /// <summary>
        /// QC 质疑
        /// </summary>
        /// <param name="inQuery"></param>
        /// <param name="_subjectVisitRepository"></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<PageOutput<ImageQuestionToBeDoneDto>> GetQCImageQuestionToBeDoneList(ImageQuestionToBeDoneQuery inQuery,
            [FromServices] IRepository<SubjectVisit> _subjectVisitRepository,
              [FromServices] IRepository<Trial> _trialRepository)
        {

            var query = _trialRepository
                .Select(t => new ImageQuestionToBeDoneDto()
                {
                    TrialId = t.Id,
                    ResearchProgramNo = t.ResearchProgramNo,
                    ExperimentName = t.ExperimentName,
                    TrialCode = t.TrialCode,
                    UrgentCount = t.SubjectVisitList.SelectMany(c => c.QCChallengeList)
                    .Where(u => u.IsClosed == false && u.LatestReplyUser.UserTypeEnum == UserTypeEnum.ClinicalResearchCoordinator && u.SubjectVisit.IsUrgent).Count(),
                    ToBeDealedCount = t.SubjectVisitList.SelectMany(c => c.QCChallengeList)
                    .Where(u => u.IsClosed == false && u.LatestReplyUser.UserTypeEnum == UserTypeEnum.ClinicalResearchCoordinator).Count(),

                });

            var result = await query.ToPagedListAsync(inQuery.PageIndex, inQuery.PageSize, inQuery.SortField, inQuery.Asc);

            return result;

        }

        #endregion


        #region IR待办

        /// <summary>
        /// IR 影像阅片 待阅量 复杂 --未完成
        /// </summary>
        /// <param name="inQuery"></param>
        /// <param name="_visitTaskRepository"></param>
        /// <param name="_trialRepository"></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<PageOutput<IRImageReadingToBeDoneDto>> GetIRImageReadingToBeDoneList(IRImageReadingToBeDoneQuery inQuery,
           [FromServices] IRepository<VisitTask> _visitTaskRepository,
             [FromServices] IRepository<Trial> _trialRepository)
        {

            var query = _trialRepository
              .Select(t => new IRImageReadingToBeDoneDto()
              {
                  TrialId = t.Id,
                  ResearchProgramNo = t.ResearchProgramNo,
                  ExperimentName = t.ExperimentName,
                  TrialCode = t.TrialCode,


                  UrgentCount = t.SubjectVisitList.SelectMany(c => c.QCChallengeList)
                  .Where(u => u.IsClosed == false && u.LatestReplyUser.UserTypeEnum == UserTypeEnum.ClinicalResearchCoordinator && u.SubjectVisit.IsUrgent).Count(),

                  UnReadCount=t.VisitTaskList.Where(t => t.DoctorUserId == _userInfo.Id  && t.ReadingTaskState != ReadingTaskState.HaveSigned && (t.TaskState == TaskState.Effect || t.TaskState == TaskState.Freeze))
                  .Where(y=>y.IsFrontTaskNeedSignButNotSign == false && (y.IsNeedClinicalDataSign == false || y.IsClinicalDataSign == true))


                  .Count(),

                  HaveSignedCount = t.VisitTaskList.Where(t => t.DoctorUserId == _userInfo.Id 
                  && (t.TaskState == TaskState.Effect || t.TaskState == TaskState.Freeze) 
                  && t.ReadingTaskState==ReadingTaskState.HaveSigned).Count()

              });


            //var query = _visitTaskRepository.Where(t=>t.DoctorUserId==_userInfo.Id && (t.TaskState == TaskState.Effect ||t.TaskState==TaskState.Freeze))
            //    .GroupBy(t=>new {t.SubjectId,t.TrialId,t.Subject.Code,t.Trial.ResearchProgramNo,t.Trial.ExperimentName,  t.BlindSubjectCode})
            //    .Select(t => new IRImageReadingToBeDoneDto()
            //    {
            //        TrialId = t.Id,
            //        ResearchProgramNo = t.ResearchProgramNo,
            //        ExperimentName = t.ExperimentName,
            //        TrialCode = t.TrialCode,
            //        UrgentCount = t.SubjectVisitList.SelectMany(c => c.QCChallengeList)
            //        .Where(u => u.IsClosed == false && u.LatestReplyUser.UserTypeEnum == UserTypeEnum.ClinicalResearchCoordinator && u.SubjectVisit.IsUrgent).Count(),
            //        ToBeDealedCount = t.SubjectVisitList.SelectMany(c => c.QCChallengeList)
            //        .Where(u => u.IsClosed == false && u.LatestReplyUser.UserTypeEnum == UserTypeEnum.ClinicalResearchCoordinator).Count(),

            //    });

            var result = await query.ToPagedListAsync(inQuery.PageIndex, inQuery.PageSize, inQuery.SortField, inQuery.Asc);

            return result;

        }


        /// <summary>
        /// IR医学反馈 
        /// </summary>
        /// <param name="inQuery"></param>

        /// <returns></returns>
        [HttpPost]
        public async Task<PageOutput<MedicalCommentsToBeDoneDto>> GetIRMedicalCommentsToBeDoneList(MedicalCommentsToBeDoneQuery inQuery,
             [FromServices] IRepository<TaskMedicalReview> _taskMedicalReviewRepository)
        {

            var query = _taskMedicalReviewRepository
                .Where(t=>t.IsClosedDialog==false)
                .GroupBy(t=>new {t.TrialId, t.Trial.ResearchProgramNo, t.Trial.ExperimentName, t.Trial.TrialCode , 
                    t.VisitTask.TrialReadingCriterionId,t.VisitTask.TrialReadingCriterion.CriterionName})
                .Select(g => new MedicalCommentsToBeDoneDto()
                {
                    TrialId = g.Key.TrialId,
                    ResearchProgramNo = g.Key.ResearchProgramNo,
                    ExperimentName = g.Key.ExperimentName,
                    TrialCode = g.Key.TrialCode,
                    CriterionName=g.Key.CriterionName,

                    UrgentCount = g.Where(u => u.VisitTask.IsUrgent  
                    && u.ReadingMedicalReviewDialogList.OrderByDescending(l=>l.CreateTime).First().UserTypeEnumInt==(int) UserTypeEnum.MIM).Count(),

                    ToBeDealedCount = g.Where(u => 
                     u.ReadingMedicalReviewDialogList.OrderByDescending(l => l.CreateTime).First().UserTypeEnumInt == (int)UserTypeEnum.MIM).Count(),

                });

            var result = await query.ToPagedListAsync(inQuery.PageIndex, inQuery.PageSize, inQuery.SortField, inQuery.Asc);

            return result;

        }




        #endregion

        #region MIM待办

        /// <summary>
        /// MIM医学反馈 
        /// </summary>
        /// <param name="inQuery"></param>

        /// <returns></returns>
        [HttpPost]
        public async Task<PageOutput<MedicalCommentsToBeDoneDto>> GetMIMMedicalCommentsToBeDoneList(MedicalCommentsToBeDoneQuery inQuery,
             [FromServices] IRepository<TaskMedicalReview> _taskMedicalReviewRepository)
        {

            var query = _taskMedicalReviewRepository
                .Where(t => t.IsClosedDialog == false)
                .GroupBy(t => new {
                    t.TrialId,
                    t.Trial.ResearchProgramNo,
                    t.Trial.ExperimentName,
                    t.Trial.TrialCode,
                    t.VisitTask.TrialReadingCriterionId,
                    t.VisitTask.TrialReadingCriterion.CriterionName
                })
                .Select(g => new MedicalCommentsToBeDoneDto()
                {
                    TrialId = g.Key.TrialId,
                    ResearchProgramNo = g.Key.ResearchProgramNo,
                    ExperimentName = g.Key.ExperimentName,
                    TrialCode = g.Key.TrialCode,
                    CriterionName = g.Key.CriterionName,

                    UrgentCount = g.Where(u => u.VisitTask.IsUrgent
                    && u.ReadingMedicalReviewDialogList.OrderByDescending(l => l.CreateTime).First().UserTypeEnumInt == (int)UserTypeEnum.IndependentReviewer).Count(),

                    ToBeDealedCount = g.Where(u =>
                     u.ReadingMedicalReviewDialogList.OrderByDescending(l => l.CreateTime).First().UserTypeEnumInt == (int)UserTypeEnum.IndependentReviewer).Count(),

                });

            var result = await query.ToPagedListAsync(inQuery.PageIndex, inQuery.PageSize, inQuery.SortField, inQuery.Asc);

            return result;

        }

        #endregion
    }
}
